@@ -3,11 +3,15 @@ import logging
 import os
 import re
 from functools import lru_cache
+from typing import Any, cast
 
 import httpx
 import pandas as pd
 from pydantic import Field
 
+# TODO Import this from typing when dropping support for Python 3.11
+from typing_extensions import TypedDict, Unpack
+
 from ixmp4.core.exceptions import ManagerApiError, PlatformNotFound
 
 from .auth import BaseAuth
@@ -17,10 +21,10 @@ from .user import User
 logger = logging.getLogger(__name__)
 
 
-class hashabledict(dict):
+class hashabledict(dict[str, Any]):
     """Hashable dict type used for caching."""
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         return hash(tuple(sorted(self.items())))
 
 
@@ -40,10 +44,14 @@ class ManagerPlatformInfo(PlatformInfo):
     accessibility: Accessibilty
 
 
+class JtiKwargs(TypedDict, total=False):
+    jti: str | None
+
+
 class ManagerConfig(Config):
     template_pattern = re.compile(r"(\{env\:(\w+)\})")
 
-    def __init__(self, url: str, auth: BaseAuth, remote: bool = False) -> None:
+    def __init__(self, url: str, auth: BaseAuth | None, remote: bool = False) -> None:
         # TODO: Find the sweet-spot for `maxsize`
         # -> a trade-off between memory usage
         # and load on the management service
@@ -74,8 +82,13 @@ class ManagerConfig(Config):
         return dsn
 
     def _uncached_request(
-        self, method: str, path: str, *args, jti: str | None = None, **kwargs
-    ):
+        self,
+        method: str,
+        path: str,
+        params: dict[str, int | None] | None = None,
+        json: dict[str, Any] | list[Any] | tuple[Any] | None = None,
+        jti: str | None = None,
+    ) -> dict[str, Any]:
         del jti
         # `jti` is only used to affect `@lru_cache`
         # if the token id changes a new cache entry will be created
@@ -84,50 +97,46 @@ class ManagerConfig(Config):
         # NOTE: since this cache is not shared amongst processes, it's efficacy
         # declines with the scale of the whole infrastructure unless counteracted
         # with increased cache size / memory usage
-        res = self.client.request(method, path, *args, **kwargs)
+        res = self.client.request(method, path, params=params, json=json)
         if res.status_code != 200:
             raise ManagerApiError(f"[{str(res.status_code)}] {res.text}")
-        return res.json()
+        # NOTE we can assume this type, might get replaced with scse-toolkit
+        return cast(dict[str, Any], res.json())
 
     def _request(
         self,
         method: str,
         path: str,
-        *args,
-        params: dict | None = None,
-        json: dict | list | tuple | None = None,
-        **kwargs,
-    ):
+        # Seems to be just that based on references
+        params: dict[str, int | None] | None = None,
+        # Seems to not be included with any references?
+        json: dict[str, Any] | list[Any] | tuple[Any] | None = None,
+        **kwargs: Unpack[JtiKwargs],
+    ) -> dict[str, Any]:
         if params is not None:
             params = hashabledict(params)
 
         if json is not None:
-            if isinstance(json, dict):
-                json = hashabledict(json)
-            else:
-                json = tuple(json)
+            json = hashabledict(json) if isinstance(json, dict) else tuple(json)
 
         logger.debug(f"Trying cache: {method} {path} {params} {json}")
-        return self._cached_request(
-            method, path, *args, params=params, json=json, **kwargs
-        )
+        return self._cached_request(method, path, params=params, json=json, **kwargs)
 
-    def fetch_platforms(self, **kwargs) -> list[ManagerPlatformInfo]:
+    def fetch_platforms(self, **kwargs: Unpack[JtiKwargs]) -> list[ManagerPlatformInfo]:
         json = self._request("GET", "/ixmp4", params={"page_size": -1}, **kwargs)
         return [ManagerPlatformInfo(**c) for c in json["results"]]
 
-    def list_platforms(self, **kwargs) -> list[ManagerPlatformInfo]:
+    def list_platforms(self, **kwargs: Unpack[JtiKwargs]) -> list[ManagerPlatformInfo]:
         platforms = self.fetch_platforms(**kwargs)
 
         for i, p in enumerate(platforms):
-            if self.remote:
-                platforms[i].dsn = p.url
-            else:
-                platforms[i].dsn = self.expand_dsn(p.dsn)
+            platforms[i].dsn = p.url if self.remote else self.expand_dsn(p.dsn)
 
         return platforms
 
-    def get_platform(self, key: str, **kwargs) -> ManagerPlatformInfo:
+    def get_platform(
+        self, key: str, **kwargs: Unpack[JtiKwargs]
+    ) -> ManagerPlatformInfo:
         for p in self.list_platforms(**kwargs):
             if p.name == key:
                 return p
@@ -138,7 +147,10 @@ class ManagerConfig(Config):
             )
 
     def fetch_user_permissions(
-        self, user: User, platform: ManagerPlatformInfo, **kwargs
+        self,
+        user: User,
+        platform: ManagerPlatformInfo,
+        **kwargs: Unpack[JtiKwargs],
     ) -> pd.DataFrame:
         if not user.is_authenticated:
             return pd.DataFrame(
@@ -159,7 +171,10 @@ class ManagerConfig(Config):
         )
 
     def fetch_group_permissions(
-        self, group_id: int, platform: ManagerPlatformInfo, **kwargs
+        self,
+        group_id: int,
+        platform: ManagerPlatformInfo,
+        **kwargs: Unpack[JtiKwargs],
     ) -> pd.DataFrame:
         json = self._request(
             "GET",
@@ -186,7 +201,10 @@ class MockManagerConfig(ManagerConfig):
         return self.platforms
 
     def fetch_user_permissions(
-        self, user: User, platform: ManagerPlatformInfo, **kwargs
+        self,
+        user: User,
+        platform: ManagerPlatformInfo,
+        **kwargs: Unpack[JtiKwargs],
     ) -> pd.DataFrame:
         pdf = self.permissions
         return pdf.where(pdf["group"].isin(user.groups)).where(
@@ -194,7 +212,10 @@ class MockManagerConfig(ManagerConfig):
         )
 
     def fetch_group_permissions(
-        self, group_id: int, platform: ManagerPlatformInfo, **kwargs
+        self,
+        group_id: int,
+        platform: ManagerPlatformInfo,
+        **kwargs: Unpack[JtiKwargs],
     ) -> pd.DataFrame:
         pdf = self.permissions
         return pdf.where(pdf["group"] == group_id).where(pdf["instance"] == platform.id)
