@@ -1,3 +1,4 @@
+from collections.abc import Iterable
 from datetime import datetime
 from typing import ClassVar
 
@@ -30,21 +31,21 @@ class Variable(BaseModelFacade):
         return self._model.created_by
 
     @property
-    def docs(self):
+    def docs(self) -> str | None:
         try:
             return self.backend.iamc.variables.docs.get(self.id).description
         except DocsModel.NotFound:
             return None
 
     @docs.setter
-    def docs(self, description):
+    def docs(self, description: str | None) -> None:
         if description is None:
             self.backend.iamc.variables.docs.delete(self.id)
         else:
             self.backend.iamc.variables.docs.set(self.id, description)
 
     @docs.deleter
-    def docs(self):
+    def docs(self) -> None:
         try:
             self.backend.iamc.variables.docs.delete(self.id)
         # TODO: silently failing
@@ -56,10 +57,7 @@ class Variable(BaseModelFacade):
 
 
 class VariableRepository(BaseFacade):
-    def create(
-        self,
-        name: str,
-    ) -> Variable:
+    def create(self, name: str) -> Variable:
         model = self.backend.iamc.variables.create(name)
         return Variable(_backend=self.backend, _model=model)
 
@@ -75,9 +73,8 @@ class VariableRepository(BaseFacade):
         return self.backend.iamc.variables.tabulate(name=name)
 
     def _get_variable_id(self, variable: str) -> int | None:
-        if variable is None:
-            return None
-        elif isinstance(variable, str):
+        # NOTE leaving this check for users without mypy
+        if isinstance(variable, str):
             obj = self.backend.iamc.variables.get(variable)
             return obj.id
         else:
@@ -115,3 +112,13 @@ class VariableRepository(BaseFacade):
             return None
         except DocsModel.NotFound:
             return None
+
+    def list_docs(
+        self, id: int | None = None, id__in: Iterable[int] | None = None
+    ) -> Iterable[str]:
+        return [
+            item.description
+            for item in self.backend.iamc.variables.docs.list(
+                dimension_id=id, dimension_id__in=id__in
+            )
+        ]
