@@ -1,13 +1,20 @@
-from typing import ClassVar, Protocol, _ProtocolMeta
+from collections.abc import Sequence
+from typing import TYPE_CHECKING, Any, ClassVar, Protocol, _ProtocolMeta
+
+from typing_extensions import Unpack
+
+if TYPE_CHECKING:
+    from ixmp4.data.db.base import EnumerateKwargs
 
 import pandas as pd
 
 from ixmp4.core.exceptions import DeletionPrevented, IxmpError, NotFound, NotUnique
-from ixmp4.data import types
 
 
 class BaseMeta(_ProtocolMeta):
-    def __init__(self, name, bases, namespace):
+    def __init__(
+        self, name: str, bases: tuple[type, ...], namespace: dict[str, Any]
+    ) -> None:
         super().__init__(name, bases, namespace)
         self.NotUnique = type(
             self.__name__ + "NotUnique",
@@ -32,43 +39,43 @@ class BaseModel(Protocol, metaclass=BaseMeta):
     NotFound: ClassVar[type[IxmpError]]
     DeletionPrevented: ClassVar[type[IxmpError]]
 
-    id: types.Integer
+    id: int
     "Unique integer id."
 
 
 class BaseRepository(Protocol):
-    def __init__(self, *args, **kwargs) -> None: ...
+    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
 
 
 class Retriever(BaseRepository, Protocol):
-    def get(self, *args, **kwargs) -> BaseModel: ...
+    def get(self, *args: Any, **kwargs: Any) -> BaseModel: ...
 
 
 class Creator(BaseRepository, Protocol):
-    def create(self, *args, **kwargs) -> BaseModel: ...
+    def create(self, *args: Any, **kwargs: Any) -> BaseModel: ...
 
 
 class Deleter(BaseRepository, Protocol):
-    def delete(self, *args, **kwargs) -> None: ...
+    def delete(self, *args: Any, **kwargs: Any) -> None: ...
 
 
 class Lister(BaseRepository, Protocol):
-    def list(self, *args, **kwargs) -> list: ...
+    def list(self, *args: Any, **kwargs: Any) -> Sequence[BaseModel]: ...
 
 
 class Tabulator(BaseRepository, Protocol):
-    def tabulate(self, *args, **kwargs) -> pd.DataFrame: ...
+    def tabulate(self, *args: Any, **kwargs: Any) -> pd.DataFrame: ...
 
 
 class Enumerator(Lister, Tabulator, Protocol):
     def enumerate(
-        self, *args, table: bool = False, **kwargs
-    ) -> list | pd.DataFrame: ...
+        self, table: bool = False, **kwargs: Unpack["EnumerateKwargs"]
+    ) -> Sequence[BaseModel] | pd.DataFrame: ...
 
 
 class BulkUpserter(BaseRepository, Protocol):
-    def bulk_upsert(self, *args, **kwargs) -> None: ...
+    def bulk_upsert(self, *args: Any, **kwargs: Any) -> None: ...
 
 
 class BulkDeleter(BaseRepository, Protocol):
-    def bulk_delete(self, *args, **kwargs) -> None: ...
+    def bulk_delete(self, *args: Any, **kwargs: Any) -> None: ...
