@@ -1,28 +1,43 @@
+from datetime import datetime
 from typing import Protocol
 
 import pandas as pd
 
-from ixmp4.data import types
+# TODO Import this from typing when dropping Python 3.11
+from typing_extensions import Unpack
 
 from .. import base
+from ..annotations import (
+    HasIdFilter,
+    HasNameFilter,
+    HasRegionFilter,
+    HasRunFilter,
+    HasUnitFilter,
+)
 from ..docs import DocsRepository
 
 
 class Variable(base.BaseModel, Protocol):
     """Variable data model."""
 
-    name: types.String
+    name: str
     "Unique name of the variable."
 
-    created_at: types.DateTime
+    created_at: datetime
     "Creation date/time. TODO"
-    created_by: types.String
+    created_by: str
     "Creator. TODO"
 
     def __str__(self) -> str:
         return f"<Variable {self.id} name={self.name}>"
 
 
+class EnumerateKwargs(HasIdFilter, HasNameFilter, total=False):
+    region: HasRegionFilter
+    run: HasRunFilter
+    unit: HasUnitFilter
+
+
 class VariableRepository(
     base.Creator,
     base.Retriever,
@@ -71,16 +86,14 @@ class VariableRepository(
         """
         ...
 
-    def list(self, *, name: str | None = None, **kwargs) -> list[Variable]:
+    def list(self, **kwargs: Unpack[EnumerateKwargs]) -> list[Variable]:
         r"""Lists variables by specified criteria.
 
         Parameters
         ----------
-        name : str
-            The name of a variable. If supplied only one result will be returned.
         \*\*kwargs: any
-            More filter parameters as specified in
-            `ixmp4.data.db.iamc.variable.filters.VariableFilter`.
+            Any filter parameters as specified in
+            `ixmp4.data.db.iamc.variable.filter.VariableFilter`.
 
         Returns
         -------
@@ -89,16 +102,14 @@ class VariableRepository(
         """
         ...
 
-    def tabulate(self, *, name: str | None = None, **kwargs) -> pd.DataFrame:
+    def tabulate(self, **kwargs: Unpack[EnumerateKwargs]) -> pd.DataFrame:
         r"""Tabulate variables by specified criteria.
 
         Parameters
         ----------
-        name : str
-            The name of a variable. If supplied only one result will be returned.
         \*\*kwargs: any
-            More filter parameters as specified in
-            `ixmp4.data.db.iamc.variable.filters.VariableFilter`.
+            Any filter parameters as specified in
+            `ixmp4.data.db.iamc.variable.filter.VariableFilter`.
 
         Returns
         -------
