@@ -1,30 +1,42 @@
+from datetime import datetime
 from typing import Protocol
 
 import pandas as pd
 
-from ixmp4.data import types
+# TODO Import this from typing when dropping Python 3.11
+from typing_extensions import Unpack
 
 from . import base
+from .annotations import (
+    HasHierarchyFilter,
+    HasIdFilter,
+    HasNameFilter,
+    IamcRegionFilter,
+)
 from .docs import DocsRepository
 
 
 class Region(base.BaseModel, Protocol):
     """Region data model."""
 
-    name: types.String
+    name: str
     "Unique name of the region."
-    hierarchy: types.String
+    hierarchy: str
     "Region hierarchy."
 
-    created_at: types.DateTime
-    "Creation date/time. TODO"
-    created_by: types.String
-    "Creator. TODO"
+    created_at: datetime
+    "Creation date/time."
+    created_by: str
+    "Creator."
 
     def __str__(self) -> str:
         return f"<Region {self.id} name={self.name}>"
 
 
+class EnumerateKwargs(HasHierarchyFilter, HasIdFilter, HasNameFilter, total=False):
+    iamc: IamcRegionFilter | bool | None
+
+
 class RegionRepository(
     base.Creator,
     base.Deleter,
@@ -57,7 +69,7 @@ class RegionRepository(
         """
         ...
 
-    def delete(self, id: int):
+    def delete(self, id: int) -> None:
         """Deletes a region.
 
         Parameters
@@ -94,11 +106,7 @@ class RegionRepository(
         """
         ...
 
-    def get_or_create(
-        self,
-        name: str,
-        hierarchy: str | None = None,
-    ) -> Region:
+    def get_or_create(self, name: str, hierarchy: str | None = None) -> Region:
         try:
             region = self.get(name)
         except Region.NotFound:
@@ -114,24 +122,14 @@ class RegionRepository(
         else:
             return region
 
-    def list(
-        self,
-        *,
-        name: str | None = None,
-        hierarchy: str | None = None,
-        **kwargs,
-    ) -> list[Region]:
+    def list(self, **kwargs: Unpack[EnumerateKwargs]) -> list[Region]:
         r"""Lists regions by specified criteria.
 
         Parameters
         ----------
-        name : str
-            The name of a region. If supplied only one result will be returned.
-        hierarchy : str
-            The hierarchy of a region.
         \*\*kwargs: any
-            More filter parameters as specified in
-            `ixmp4.data.db.region.filters.RegionFilter`.
+            Any filter parameters as specified in
+            `ixmp4.data.db.region.filter.RegionFilter`.
 
         Returns
         -------
@@ -140,24 +138,14 @@ class RegionRepository(
         """
         ...
 
-    def tabulate(
-        self,
-        *,
-        name: str | None = None,
-        hierarchy: str | None = None,
-        **kwargs,
-    ) -> pd.DataFrame:
+    def tabulate(self, **kwargs: Unpack[EnumerateKwargs]) -> pd.DataFrame:
         r"""Tabulate regions by specified criteria.
 
         Parameters
         ----------
-        name : str
-            The name of a region. If supplied only one result will be returned.
-        hierarchy : str
-            The hierarchy of a region.
         \*\*kwargs: any
-            More filter parameters as specified in
-            `ixmp4.data.db.region.filters.RegionFilter`.
+            Any filter parameters as specified in
+            `ixmp4.data.db.region.filter.RegionFilter`.
 
         Returns
         -------
