@@ -1,50 +1,73 @@
-from typing import ClassVar, Protocol
+from datetime import datetime
+from typing import TYPE_CHECKING, ClassVar, Protocol
 
 import pandas as pd
 
-from ixmp4.core.exceptions import IxmpError, NoDefaultRunVersion
-from ixmp4.data import types
+# TODO Import this from typing when dropping Python 3.11
+from typing_extensions import Unpack
+
+from ixmp4.core.exceptions import (
+    IxmpError,
+    NoDefaultRunVersion,
+    RunIsLocked,
+    RunLockRequired,
+)
 
 from . import base
+from .annotations import HasRunFilter, IamcRunFilter
+
+if TYPE_CHECKING:
+    from . import Model, Scenario
 
 
 class Run(base.BaseModel, Protocol):
     """Model run data model."""
 
     NoDefaultVersion: ClassVar[type[IxmpError]] = NoDefaultRunVersion
+    IsLocked: ClassVar[type[IxmpError]] = RunIsLocked
+    LockRequired: ClassVar[type[IxmpError]] = RunLockRequired
 
-    model__id: types.Integer
+    model__id: int
     "Foreign unique integer id of the model."
-    model: types.Mapped
+    model: "Model"
     "Associated model."
 
-    scenario__id: types.Integer
+    scenario__id: int
     "Foreign unique integer id of the scenario."
-    scenario: types.Mapped
+    scenario: "Scenario"
     "Associated scenario."
 
-    version: types.Integer
+    version: int
     "Run version."
-    is_default: types.Boolean
+    is_default: bool
     "`True` if this is the default run version."
 
+    created_at: datetime
+    created_by: str
+
+    updated_at: datetime
+    updated_by: str
+
+    lock_transaction: int | None
+
     def __str__(self) -> str:
         return f"<Run {self.id} model={self.model.name} \
             scenario={self.scenario.name} version={self.version} \
             is_default={self.is_default}>"
 
 
+class EnumerateKwargs(HasRunFilter, total=False):
+    iamc: IamcRunFilter | bool | None
+
+
 class RunRepository(
     base.Creator,
+    base.Deleter,
     base.Retriever,
     base.Enumerator,
     Protocol,
 ):
-    def create(
-        self,
-        model_name: str,
-        scenario_name: str,
-    ) -> Run:
+    def create(self, model_name: str, scenario_name: str) -> Run:
         """Creates a run with an incremented version number or version=1 if no versions
         exist. Will automatically create the models and scenarios if they don't exist
         yet.
@@ -63,12 +86,22 @@ class RunRepository(
         """
         ...
 
-    def get(
-        self,
-        model_name: str,
-        scenario_name: str,
-        version: int,
-    ) -> Run:
+    def delete(self, id: int) -> None:
+        """Deletes a run and **all associated iamc, optimization and meta data**.
+
+        Parameters
+        ----------
+        id : int
+            The unique integer id of the run.
+
+        Raises
+        ------
+        :class:`ixmp4.data.abstract.Run.NotFound`:
+            If the run with `id` does not exist.
+        """
+        ...
+
+    def get(self, model_name: str, scenario_name: str, version: int) -> Run:
         """Retrieves a run.
 
         Parameters
@@ -92,11 +125,7 @@ class RunRepository(
         """
         ...
 
-    def get_or_create(
-        self,
-        model_name: str,
-        scenario_name: str,
-    ) -> Run:
+    def get_or_create(self, model_name: str, scenario_name: str) -> Run:
         """Tries to retrieve a run's default version
         and creates it if it was not found.
 
@@ -117,11 +146,7 @@ class RunRepository(
         except Run.NoDefaultVersion:
             return self.create(model_name, scenario_name)
 
-    def get_default_version(
-        self,
-        model_name: str,
-        scenario_name: str,
-    ) -> Run:
+    def get_default_version(self, model_name: str, scenario_name: str) -> Run:
         """Retrieves a run's default version.
 
         Parameters
@@ -143,23 +168,34 @@ class RunRepository(
         """
         ...
 
-    def list(
-        self,
-        *,
-        version: int | None = None,
-        **kwargs,
-    ) -> list[Run]:
+    def get_by_id(self, id: int) -> Run:
+        """Retrieves a Run by its id.
+
+        Parameters
+        ----------
+        id : int
+            Unique integer id.
+
+        Raises
+        ------
+        :class:`ixmp4.data.abstract.Run.NotFound`.
+            If the Run with `id` does not exist.
+
+        Returns
+        -------
+        :class:`ixmp4.data.abstract.Run`:
+            The retrieved Run.
+        """
+        ...
+
+    def list(self, **kwargs: Unpack[EnumerateKwargs]) -> list[Run]:
         r"""Lists runs by specified criteria.
 
         Parameters
         ----------
-        version : int
-            The run's version.
-        default_only : bool
-            True by default. This function will return default runs only if true.
         \*\*kwargs: any
-            More filter parameters as specified in
-            `ixmp4.data.db.run.filters.RunFilter`.
+            Any filter parameters as specified in
+            `ixmp4.data.db.run.filter.RunFilter`.
 
         Returns
         -------
@@ -168,23 +204,14 @@ class RunRepository(
         """
         ...
 
-    def tabulate(
-        self,
-        *,
-        version: int | None = None,
-        **kwargs,
-    ) -> pd.DataFrame:
+    def tabulate(self, **kwargs: Unpack[EnumerateKwargs]) -> pd.DataFrame:
         r"""Tabulate runs by specified criteria.
 
         Parameters
         ----------
-        version : int
-            The run's version.
-        default_only : bool
-            True by default. This function will return default runs only if true.
         \*\*kwargs: any
-            More filter parameters as specified in
-            `ixmp4.data.db.run.filters.RunFilter`.
+            Any filter parameters as specified in
+            `ixmp4.data.db.run.filter.RunFilter`.
 
         Returns
         -------
@@ -230,3 +257,81 @@ class RunRepository(
 
         """
         ...
+
+    def revert(self, id: int, transaction__id: int) -> None:
+        """Reverts run data to a specific `transaction__id`.
+
+        Parameters
+        ----------
+        id : int
+            Unique integer id.
+        transaction__id : int
+            Id of the transaction to revert to.
+
+        Raises
+        ------
+        :class:`ixmp4.data.abstract.Run.NotFound`:
+            If no run with the `id` exists.
+        """
+        ...
+
+    def lock(self, id: int) -> Run:
+        """Locks a run at the current transaction (via `transaction__id`).
+
+        Parameters
+        ----------
+        id : int
+            Unique integer id.
+
+        Raises
+        ------
+        :class:`ixmp4.data.abstract.Run.NotFound`:
+            If no run with the `id` exists.
+        :class:`ixmp4.data.abstract.Run.IsLocked`:
+            If the run is already locked.
+
+        """
+        ...
+
+    def unlock(self, id: int) -> Run:
+        """Locks a run at the current transaction (via `transaction__id`).
+
+        Parameters
+        ----------
+        id : int
+            Unique integer id.
+
+        Raises
+        ------
+        :class:`ixmp4.data.abstract.Run.NotFound`:
+            If no run with the `id` exists.
+        """
+        ...
+
+    def clone(
+        self,
+        run_id: int,
+        model_name: str | None = None,
+        scenario_name: str | None = None,
+        keep_solution: bool = True,
+    ) -> Run:
+        """Clone all data from one run to a new one.
+
+        Parameters
+        ----------
+        run_id: int
+            The unique integer id of the base run.
+        model_name: str | None
+            The new name of the model used in the new run, optional.
+        scenario_name: str | None
+            The new name of the scenario used in the new run, optional.
+        keep_solution: bool
+            Whether to keep the solution data from the base run. Optional, defaults to
+            `True`.
+
+        Returns
+        -------
+        :class:`ixmp4.data.abstract.Run`:
+            The clone of the base run.
+        """
+        ...
