@@ -1,4 +1,7 @@
-from typing import ClassVar, Type
+from collections.abc import Iterable
+from typing import ClassVar
+
+import pandas as pd
 
 from ixmp4.data import abstract
 
@@ -22,7 +25,7 @@ class DocsRepository(
     base.Enumerator[Docs],
     abstract.DocsRepository,
 ):
-    model_class: Type[Docs]
+    model_class: type[Docs]
     enumeration_method = "GET"
 
     def get(self, dimension_id: int) -> Docs:
@@ -38,8 +41,20 @@ class DocsRepository(
         )
         return Docs(**res)
 
-    def list(self, *, dimension_id: int | None = None) -> list[Docs]:
-        return super()._list(params={"dimension_id": dimension_id})
+    # NOTE This is not used anywhere, but without it, mypy complains that the base
+    # definitions of enumerate() are incompatible
+    def enumerate(self, dimension_id: int | None = None) -> list[Docs] | pd.DataFrame:
+        return super().enumerate(dimension_id=dimension_id)
+
+    def list(
+        self,
+        *,
+        dimension_id: int | None = None,
+        dimension_id__in: Iterable[int] | None = None,
+    ) -> list[Docs]:
+        return super()._list(
+            params={"dimension_id": dimension_id, "dimension_id__in": dimension_id__in}
+        )
 
     def delete(self, dimension_id: int) -> None:
         super().delete(dimension_id)
