@@ -2,14 +2,13 @@ import logging
 from concurrent.futures import ThreadPoolExecutor
 
 import httpx
-import pandas as pd
-from fastapi.testclient import TestClient
 
 from ixmp4.conf import settings
-from ixmp4.conf.auth import BaseAuth, SelfSignedAuth, User
-from ixmp4.conf.manager import ManagerPlatformInfo, MockManagerConfig, PlatformInfo
+from ixmp4.conf.auth import BaseAuth, SelfSignedAuth
+from ixmp4.conf.manager import ManagerPlatformInfo, PlatformInfo
 from ixmp4.core.exceptions import ImproperlyConfigured, UnknownApiError
 from ixmp4.data.api import (
+    CheckpointRepository,
     DataPointRepository,
     EquationRepository,
     IndexSetRepository,
@@ -26,8 +25,7 @@ from ixmp4.data.api import (
     UnitRepository,
     VariableRepository,
 )
-from ixmp4.server import app, v1
-from ixmp4.server.rest import APIInfo, deps
+from ixmp4.server.rest import APIInfo
 
 from .base import Backend
 
@@ -55,7 +53,7 @@ class RestBackend(Backend):
                 logger.info("Platform notice: >\n" + info.notice)
         self.create_repositories()
 
-    def make_client(self, rest_url: str, auth: BaseAuth | None):
+    def make_client(self, rest_url: str, auth: BaseAuth | None) -> None:
         auth = self.get_auth(rest_url, auth)
 
         self.client = httpx.Client(
@@ -65,7 +63,9 @@ class RestBackend(Backend):
             auth=auth,
         )
 
-    def get_auth(self, rest_url: str, override_auth: BaseAuth | None) -> BaseAuth:
+    def get_auth(
+        self, rest_url: str, override_auth: BaseAuth | None
+    ) -> BaseAuth | None:
         root = httpx.get(rest_url, follow_redirects=True)
         if root.status_code != 200:
             logger.error("Root API response not OK: " + root.text)
@@ -109,7 +109,7 @@ class RestBackend(Backend):
         else:
             return override_auth
 
-    def create_repositories(self):
+    def create_repositories(self) -> None:
         self.iamc.datapoints = DataPointRepository(self)
         self.iamc.timeseries = TimeSeriesRepository(self)
         self.iamc.variables = VariableRepository(self)
@@ -125,59 +125,4 @@ class RestBackend(Backend):
         self.runs = RunRepository(self)
         self.scenarios = ScenarioRepository(self)
         self.units = UnitRepository(self)
-
-
-test_platform = ManagerPlatformInfo(
-    id=1,
-    management_group=1,
-    access_group=1,
-    accessibility=ManagerPlatformInfo.Accessibilty.PRIVATE,
-    slug="test",
-    notice="Welcome to the test platform.",
-    dsn="http://testserver/v1/:test:/",
-    url="http://testserver/v1/:test:/",
-)
-
-test_user = User(id=-1, username="@unknown", is_superuser=True, is_verified=True)
-
-test_permissions = pd.DataFrame(
-    [], columns=["id", "instance", "group", "access_type", "model"]
-)
-
-mock_manager = MockManagerConfig([test_platform], test_permissions)
-
-
-class RestTestBackend(RestBackend):
-    def __init__(self, db_backend, *args, **kwargs) -> None:
-        self.db_backend = db_backend
-        self.auth_params = (test_user, mock_manager, test_platform)
-        super().__init__(
-            test_platform, SelfSignedAuth(settings.secret_hs256), *args, **kwargs
-        )
-
-    def make_client(self, rest_url: str, auth: BaseAuth):
-        self.client = TestClient(
-            app=app,
-            base_url=rest_url,
-            raise_server_exceptions=False,
-        )
-
-        app.dependency_overrides[deps.validate_token] = deps.do_not_validate_token
-        v1.dependency_overrides[deps.validate_token] = deps.do_not_validate_token
-
-        app.dependency_overrides[deps.get_backend] = deps.get_test_backend_dependency(
-            self.db_backend, self.auth_params
-        )
-        v1.dependency_overrides[deps.get_backend] = deps.get_test_backend_dependency(
-            self.db_backend, self.auth_params
-        )
-
-    def close(self):
-        self.client.close()
-        self.executor.shutdown(cancel_futures=True)
-
-    def setup(self):
-        pass
-
-    def teardown(self):
-        pass
+        self.checkpoints = CheckpointRepository(self)
