@@ -1,5 +1,6 @@
 from ixmp4.conf.base import PlatformInfo
 from ixmp4.data.abstract import (
+    CheckpointRepository,
     DataPointRepository,
     EquationRepository,
     IndexSetRepository,
@@ -12,6 +13,7 @@ from ixmp4.data.abstract import (
     ScalarRepository,
     ScenarioRepository,
     TableRepository,
+    TimeSeries,
     TimeSeriesRepository,
     UnitRepository,
     VariableRepository,
@@ -20,7 +22,7 @@ from ixmp4.data.abstract import (
 
 class IamcSubobject(object):
     datapoints: DataPointRepository
-    timeseries: TimeSeriesRepository
+    timeseries: TimeSeriesRepository[TimeSeries]
     variables: VariableRepository
 
 
@@ -43,6 +45,7 @@ class Backend(object):
     runs: RunRepository
     scenarios: ScenarioRepository
     units: UnitRepository
+    checkpoints: CheckpointRepository
 
     def __init__(self, info: PlatformInfo) -> None:
         self.info = info
@@ -53,5 +56,5 @@ class Backend(object):
         """Closes the connection to the database."""
         ...
 
-    def __str__(self):
+    def __str__(self) -> str:
         return f"<Backend {self.info}>"
