@@ -1,11 +1,12 @@
 import logging
+from collections.abc import Generator
 from contextlib import contextmanager
 from functools import lru_cache
-from typing import Generator
 
 from sqlalchemy.engine import Engine, create_engine
+from sqlalchemy.orm import configure_mappers
 from sqlalchemy.orm.session import sessionmaker
-from sqlalchemy.pool import NullPool, StaticPool
+from sqlalchemy.pool import NullPool
 
 from ixmp4.conf.base import PlatformInfo
 from ixmp4.conf.manager import ManagerConfig, ManagerPlatformInfo
@@ -13,6 +14,7 @@ from ixmp4.conf.user import User
 from ixmp4.core.exceptions import ProgrammingError
 from ixmp4.data.db import (
     BaseModel,
+    CheckpointRepository,
     DataPointRepository,
     EquationRepository,
     IndexSetRepository,
@@ -26,6 +28,7 @@ from ixmp4.data.db import (
     ScenarioRepository,
     TableRepository,
     TimeSeriesRepository,
+    TransactionRepository,
     UnitRepository,
     VariableRepository,
 )
@@ -43,7 +46,7 @@ logger = logging.getLogger(__name__)
 
 @lru_cache()
 def cached_create_engine(dsn: str) -> Engine:
-    return create_engine(dsn, poolclass=NullPool)
+    return create_engine(dsn, poolclass=NullPool, max_identifier_length=63)
 
 
 class IamcSubobject(BaseIamcSubobject):
@@ -71,6 +74,9 @@ class SqlAlchemyBackend(Backend):
     runs: RunRepository
     scenarios: ScenarioRepository
     units: UnitRepository
+    checkpoints: CheckpointRepository
+    transactions: TransactionRepository
+
     Session = sessionmaker(autocommit=False, autoflush=False, future=True)
     auth_context: AuthorizationContext | None = None
     event_handler: SqlaEventHandler
@@ -78,12 +84,14 @@ class SqlAlchemyBackend(Backend):
     def __init__(self, info: PlatformInfo) -> None:
         super().__init__(info)
         logger.info(f"Creating database engine for platform '{info.name}'.")
+        configure_mappers()
+
         dsn = self.check_dsn(info.dsn)
         self.make_engine(dsn)
         self.make_repositories()
         self.event_handler = SqlaEventHandler(self)
 
-    def check_dsn(self, dsn: str):
+    def check_dsn(self, dsn: str) -> str:
         if dsn.startswith("postgresql://"):
             logger.debug(
                 "Replacing the platform dsn prefix to use the new `psycopg` driver."
@@ -91,11 +99,11 @@ class SqlAlchemyBackend(Backend):
             dsn = dsn.replace("postgresql://", "postgresql+psycopg://")
         return dsn
 
-    def make_engine(self, dsn: str):
+    def make_engine(self, dsn: str) -> None:
         self.engine = cached_create_engine(dsn)
         self.session = self.Session(bind=self.engine)
 
-    def make_repositories(self):
+    def make_repositories(self) -> None:
         self.iamc.datapoints = DataPointRepository(self)
         self.iamc.timeseries = TimeSeriesRepository(self)
         self.iamc.variables = VariableRepository(self)
@@ -111,6 +119,8 @@ class SqlAlchemyBackend(Backend):
         self.runs = RunRepository(self)
         self.scenarios = ScenarioRepository(self)
         self.units = UnitRepository(self)
+        self.checkpoints = CheckpointRepository(self)
+        self.transactions = TransactionRepository(self)
 
     @contextmanager
     def auth(
@@ -127,49 +137,19 @@ class SqlAlchemyBackend(Backend):
         yield self.auth_context
         self.auth_context = None
 
-    def _create_all(self):
+    def _create_all(self) -> None:
         BaseModel.metadata.create_all(bind=self.engine)
 
-    def _drop_all(self):
+    def _drop_all(self) -> None:
         BaseModel.metadata.drop_all(bind=self.engine, checkfirst=True)
 
-    def setup(self):
+    def setup(self) -> None:
         self._create_all()
 
-    def teardown(self):
+    def teardown(self) -> None:
         self.session.rollback()
         self._drop_all()
-        self.engine = None
-        self.session = None
 
-    def close(self):
+    def close(self) -> None:
         self.session.close()
         self.engine.dispose()
-
-
-class SqliteTestBackend(SqlAlchemyBackend):
-    def __init__(self, *args, **kwargs) -> None:
-        super().__init__(
-            *args,
-            **kwargs,
-        )
-
-    def make_engine(self, dsn: str):
-        self.engine = create_engine(
-            dsn,
-            connect_args={"check_same_thread": False},
-            poolclass=StaticPool,
-        )
-        self.session = self.Session(bind=self.engine)
-
-
-class PostgresTestBackend(SqlAlchemyBackend):
-    def __init__(self, *args, **kwargs) -> None:
-        super().__init__(
-            *args,
-            **kwargs,
-        )
-
-    def make_engine(self, dsn: str):
-        self.engine = create_engine(dsn, poolclass=NullPool)
-        self.session = self.Session(bind=self.engine)
