@@ -2,33 +2,30 @@ from __future__ import annotations
 
 import logging
 import sqlite3
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    ClassVar,
-    Generic,
-    Iterable,
-    Iterator,
-    Tuple,
-    TypeVar,
-    cast,
-)
+from collections.abc import Callable, Iterable, Iterator, Mapping
+from datetime import datetime
+from typing import TYPE_CHECKING, Any, ClassVar, Generic, Protocol, TypeVar, cast
 
 import numpy as np
 import pandas as pd
-from sqlalchemy import event, text
+from sqlalchemy import TextClause, event, text
 from sqlalchemy.engine import Engine
 from sqlalchemy.engine.interfaces import Dialect
 from sqlalchemy.exc import IntegrityError, NoResultFound
 from sqlalchemy.ext.compiler import compiles
-from sqlalchemy.orm import Bundle, DeclarativeBase, declared_attr
+from sqlalchemy.orm import Bundle, DeclarativeBase
 from sqlalchemy.orm.session import Session
 from sqlalchemy.sql.schema import Identity, MetaData
 
+# TODO Import this from typing when dropping Python 3.11
+from typing_extensions import NotRequired, TypedDict, Unpack
+
 from ixmp4 import db
 from ixmp4.core.exceptions import Forbidden, IxmpError, ProgrammingError
 from ixmp4.data import abstract, types
+from ixmp4.data.abstract.annotations import (
+    TabulateVersionsKwargs as TabulateVersionsKwargs,
+)
 from ixmp4.db import filters
 
 if TYPE_CHECKING:
@@ -38,7 +35,9 @@ logger = logging.getLogger(__name__)
 
 
 @event.listens_for(Engine, "connect")
-def set_sqlite_pragma(dbapi_connection, connection_record):
+def set_sqlite_pragma(
+    dbapi_connection: sqlite3.Connection, connection_record: Any
+) -> None:
     if isinstance(dbapi_connection, sqlite3.Connection):
         cursor = dbapi_connection.cursor()
         cursor.execute("PRAGMA foreign_keys=ON")
@@ -46,7 +45,7 @@ def set_sqlite_pragma(dbapi_connection, connection_record):
 
 
 @compiles(Identity, "sqlite")
-def visit_identity(element, compiler, **kwargs):
+def visit_identity(element: Any, compiler: Any, **kwargs: Any) -> TextClause:
     return text("")
 
 
@@ -55,21 +54,16 @@ class BaseModel(DeclarativeBase):
     NotUnique: ClassVar[type[IxmpError]]
     DeletionPrevented: ClassVar[type[IxmpError]]
 
-    table_prefix: str = ""
     updateable_columns: ClassVar[list[str]] = []
 
-    @declared_attr.directive
-    def __tablename__(cls: "BaseModel") -> str:
-        return str(cls.table_prefix + cls.__name__.lower())
-
-    id: types.Integer = db.Column(
+    id: types.Id = db.Column(
         db.Integer,
         Identity(always=False, on_null=True, start=1, increment=1),
         primary_key=True,
         info={"skip_autogenerate": True},
     )
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.__class__.__name__
 
 
@@ -83,6 +77,7 @@ BaseModel.metadata = MetaData(
     }
 )
 
+SelectType = TypeVar("SelectType", bound=db.sql.Select[tuple[BaseModel, ...]])
 
 ModelType = TypeVar("ModelType", bound=BaseModel)
 
@@ -91,10 +86,10 @@ class BaseRepository(Generic[ModelType]):
     backend: "SqlAlchemyBackend"
     session: Session
     dialect: Dialect
-    bundle: Bundle
+    bundle: Bundle[Any]
     model_class: type[ModelType]
 
-    def __init__(self, backend: "SqlAlchemyBackend", *args, **kwargs) -> None:
+    def __init__(self, backend: "SqlAlchemyBackend") -> None:
         self.backend = backend
         self.session = backend.session
         self.engine = backend.engine
@@ -104,22 +99,40 @@ class BaseRepository(Generic[ModelType]):
         else:
             raise ProgrammingError("Database session is closed.")
 
-        self.bundle: Bundle = Bundle(
+        self.bundle: Bundle[Any] = Bundle(
             self.model_class.__name__, *db.utils.get_columns(self.model_class).values()
         )
-        super().__init__(*args, **kwargs)
+
+        super().__init__()
 
 
 class Retriever(BaseRepository[ModelType], abstract.Retriever):
-    def get(self, *args, **kwargs) -> ModelType:
+    def get(self, *args: Any, **kwargs: Any) -> ModelType:
         raise NotImplementedError
 
 
+class CreateKwargs(TypedDict, total=False):
+    dimension_id: int
+    description: str
+    name: str
+    model_name: str
+    scenario_name: str
+    hierarchy: str
+    run__id: int
+    key: str
+    value: abstract.annotations.PrimitiveTypes
+    parameters: Mapping[str, Any]
+    run_id: int
+    unit_name: str | None
+    column_names: list[str] | None
+    constrained_to_indexsets: list[str] | None
+
+
 class Creator(BaseRepository[ModelType], abstract.Creator):
-    def add(self, *args, **kwargs) -> ModelType:
+    def add(self, *args: Any, **kwargs: Any) -> ModelType:
         raise NotImplementedError
 
-    def create(self, *args, **kwargs) -> ModelType:
+    def create(self, *args: Any, **kwargs: Unpack[CreateKwargs]) -> ModelType:
         model = self.add(*args, **kwargs)
         try:
             self.session.commit()
@@ -130,28 +143,56 @@ class Creator(BaseRepository[ModelType], abstract.Creator):
         return model
 
 
-class Deleter(BaseRepository[ModelType]):
-    def delete(self, id: int):
-        exc: db.sql.Delete = db.delete(self.model_class).where(
-            self.model_class.id == id
-        )
+class Deleter(BaseRepository[ModelType], abstract.Deleter):
+    def delete(self, id: int) -> None:
+        exc = db.select(self.model_class).where(self.model_class.id == id)
 
         try:
-            self.session.execute(
-                exc, execution_options={"synchronize_session": "fetch"}
-            )
-            self.session.commit()
+            obj = self.session.execute(exc).scalar_one()
         except NoResultFound:
-            raise self.model_class.NotFound
+            raise self.model_class.NotFound(id=id)
+
+        try:
+            self.session.delete(obj)
+            self.session.commit()
         except IntegrityError:
+            self.session.rollback()
             raise self.model_class.DeletionPrevented
 
 
-class Selecter(BaseRepository[ModelType]):
-    filter_class: type[filters.BaseFilter]
+class CheckAccessKwargs(TypedDict, total=False):
+    run: abstract.annotations.HasRunFilter
+    is_default: bool | None
+    default_only: bool | None
 
-    def check_access(self, ids: set[int], access_type: str = "view", **kwargs):
-        exc = self.select(
+
+class SelectCountKwargs(abstract.HasNameFilter, total=False):
+    default_only: bool | None
+    dimension_id: int | None
+    dimension_id__in: Iterable[int] | None
+    id__in: set[int]
+    is_default: bool | None
+    join_parameters: bool | None
+    join_runs: bool | None
+    iamc: abstract.annotations.IamcFilterAlias
+    model: abstract.annotations.HasModelFilter
+    region: abstract.annotations.HasRegionFilter
+    run: abstract.annotations.HasRunFilter
+    scenario: abstract.annotations.HasScenarioFilter
+    unit: abstract.annotations.HasUnitFilter
+    variable: abstract.annotations.HasVariableFilter
+
+
+class Selecter(BaseRepository[ModelType]):
+    filter_class: type[filters.BaseFilter] | None
+
+    def check_access(
+        self,
+        ids: set[int],
+        access_type: str = "view",
+        **kwargs: Unpack[CheckAccessKwargs],
+    ) -> None:
+        exc = self.select_for_count(
             _exc=db.select(db.func.count()).select_from(self.model_class),
             id__in=ids,
             _access_type=access_type,
@@ -166,25 +207,55 @@ class Selecter(BaseRepository[ModelType]):
             )
             raise Forbidden(f"Permission check failed for access type '{access_type}'.")
 
-    def join_auth(self, exc: db.sql.Select) -> db.sql.Select:
+    SelectAuthType = TypeVar("SelectAuthType", bound=db.sql.Select[tuple[Any]])
+
+    def join_auth(self, exc: SelectAuthType) -> SelectAuthType:
         return exc
 
-    def apply_auth(self, exc: db.sql.Select, access_type: str) -> db.sql.Select:
+    def apply_auth(self, exc: SelectAuthType, access_type: str) -> SelectAuthType:
         if self.backend.auth_context is not None:
             if not self.backend.auth_context.is_managed:
                 exc = self.join_auth(exc)
             exc = self.backend.auth_context.apply(access_type, exc)
         return exc
 
+    def select_for_count(
+        self,
+        _exc: db.sql.Select[tuple[int]],
+        _filter: filters.BaseFilter | None = None,
+        _skip_filter: bool = False,
+        _access_type: str = "view",
+        **kwargs: Unpack[SelectCountKwargs],
+    ) -> db.sql.Select[tuple[int]]:
+        if self.filter_class is None:
+            cls_name = self.__class__.__name__
+            raise NotImplementedError(
+                f"Provide `{cls_name}.filter_class` or reimplement `{cls_name}.select`."
+            )
+
+        _exc = self.apply_auth(_exc, _access_type)
+        if _filter is not None and not _skip_filter:
+            filter_instance = _filter
+            _exc = filter_instance.join(_exc, session=self.session)
+            _exc = filter_instance.apply(_exc, self.model_class, self.session)
+        elif not _skip_filter:
+            kwarg_filter: filters.BaseFilter = self.filter_class(**kwargs)
+            _exc = kwarg_filter.join(_exc, session=self.session)
+            _exc = kwarg_filter.apply(_exc, self.model_class, self.session)
+        return _exc
+
     def select(
         self,
         _filter: filters.BaseFilter | None = None,
-        _exc: db.sql.Select | None = None,
+        _exc: db.sql.Select[tuple[ModelType]] | None = None,
         _access_type: str = "view",
-        _post_filter: Callable[[db.sql.Select], db.sql.Select] | None = None,
+        _post_filter: Callable[
+            [db.sql.Select[tuple[ModelType]]], db.sql.Select[tuple[ModelType]]
+        ]
+        | None = None,
         _skip_filter: bool = False,
-        **kwargs,
-    ) -> db.sql.Select:
+        **kwargs: Any,
+    ) -> db.sql.Select[tuple[ModelType]]:
         if self.filter_class is None:
             cls_name = self.__class__.__name__
             raise NotImplementedError(
@@ -197,12 +268,11 @@ class Selecter(BaseRepository[ModelType]):
         _exc = self.apply_auth(_exc, _access_type)
 
         if _filter is not None and not _skip_filter:
-            # for some reason checkers resolve the type of `_filter` to `Unknown`
-            filter_instance: filters.BaseFilter = _filter
+            filter_instance = _filter
             _exc = filter_instance.join(_exc, session=self.session)
             _exc = filter_instance.apply(_exc, self.model_class, self.session)
         elif not _skip_filter:
-            kwarg_filter = self.filter_class(**kwargs)
+            kwarg_filter: filters.BaseFilter = self.filter_class(**kwargs)
             _exc = kwarg_filter.join(_exc, session=self.session)
             _exc = kwarg_filter.apply(_exc, self.model_class, self.session)
 
@@ -211,62 +281,95 @@ class Selecter(BaseRepository[ModelType]):
         return _exc
 
 
-class Lister(Selecter[ModelType]):
-    def list(self, *args, **kwargs) -> list[ModelType]:
-        _exc = self.select(*args, **kwargs)
-        _exc = _exc.order_by(self.model_class.id.asc())
-        result = self.session.execute(_exc).scalars().all()
-        return list(result)
-
-
-class Tabulator(Selecter[ModelType]):
-    def tabulate(
-        self,
-        *args,
-        _raw: bool = False,
-        **kwargs,
-    ) -> pd.DataFrame:
-        _exc = self.select(*args, **kwargs)
-        _exc = _exc.order_by(self.model_class.id.asc())
-
+class QueryMixin(BaseRepository[ModelType]):
+    def tabulate_query(self, exc: db.sql.Select[Any]) -> pd.DataFrame:
         if self.session.bind is not None:
             with self.engine.connect() as con:
-                return pd.read_sql(_exc, con=con).replace([np.nan], [None])
+                return pd.read_sql(exc, con=con).replace({np.nan: None})
         else:
             raise ProgrammingError("Database session is closed.")
 
+    def list_query(self, exc: db.sql.Select[Any]) -> list[Any]:
+        result = self.session.execute(exc).scalars().all()
+        return list(result)
 
-class Enumerator(Lister[ModelType], Tabulator[ModelType]):
+
+class Lister(QueryMixin[ModelType], Selecter[ModelType], abstract.Lister):
+    def list(self, *args: Any, **kwargs: Any) -> list[ModelType]:
+        _exc = self.select(*args, **kwargs)
+        _exc = _exc.order_by(self.model_class.id.asc())
+        return self.list_query(_exc)
+
+
+class Tabulator(QueryMixin[ModelType], Selecter[ModelType], abstract.Tabulator):
+    def tabulate(self, *args: Any, _raw: bool = False, **kwargs: Any) -> pd.DataFrame:
+        _exc = self.select(*args, **kwargs)
+        _exc = _exc.order_by(self.model_class.id.asc())
+
+        return self.tabulate_query(_exc)
+
+
+class PaginateKwargs(TypedDict):
+    _filter: filters.BaseFilter
+    join_parameters: NotRequired[bool | None]
+    join_runs: NotRequired[bool | None]
+    join_run_index: NotRequired[bool | None]
+    table: bool
+
+
+class EnumerateKwargs(TypedDict):
+    _filter: filters.BaseFilter
+    join_parameters: NotRequired[bool | None]
+    join_runs: NotRequired[bool | None]
+    join_run_index: NotRequired[bool | None]
+    _post_filter: Callable[
+        [db.sql.Select[tuple[ModelType]]], db.sql.Select[tuple[ModelType]]
+    ]
+
+
+class CountKwargs(abstract.HasNameFilter, total=False):
+    dimension_id: int | None
+    dimension_id__in: Iterable[int] | None
+    _filter: filters.BaseFilter
+    join_parameters: bool | None
+    join_runs: bool | None
+    iamc: abstract.annotations.IamcFilterAlias
+    model: abstract.annotations.HasModelFilter
+    region: abstract.annotations.HasRegionFilter
+    run: abstract.annotations.HasRunFilter
+    scenario: abstract.annotations.HasScenarioFilter
+    unit: abstract.annotations.HasUnitFilter
+    variable: abstract.annotations.HasVariableFilter
+
+
+class Enumerator(Lister[ModelType], Tabulator[ModelType], abstract.Enumerator):
     def enumerate(
-        self, *args, table: bool = False, **kwargs
+        self, table: bool = False, **kwargs: Unpack[EnumerateKwargs]
     ) -> list[ModelType] | pd.DataFrame:
-        if table:
-            return self.tabulate(*args, **kwargs)
-        else:
-            return self.list(*args, **kwargs)
+        return self.tabulate(**kwargs) if table else self.list(**kwargs)
 
     def paginate(
-        self,
-        *args,
-        limit: int = 1000,
-        offset: int = 0,
-        **kwargs,
+        self, limit: int = 1000, offset: int = 0, **kwargs: Unpack[PaginateKwargs]
     ) -> list[ModelType] | pd.DataFrame:
         return self.enumerate(
-            *args, **kwargs, _post_filter=lambda e: e.offset(offset).limit(limit)
+            **kwargs, _post_filter=lambda e: e.offset(offset).limit(limit)
         )
 
-    def count(
-        self,
-        **kwargs,
-    ) -> int:
-        _exc = self.select(
+    def count(self, **kwargs: Unpack[CountKwargs]) -> int:
+        _exc = self.select_for_count(
             _exc=db.select(db.func.count(self.model_class.id.distinct())),
             **kwargs,
         )
         return self.session.execute(_exc).scalar_one()
 
 
+class TransactionProtocol(Protocol):
+    """Protocol class to mock the type of sqlalchemy_continuums"""
+
+    id: int
+    issued_at: datetime
+
+
 class BulkOperator(Tabulator[ModelType]):
     merge_suffix: str = "_y"
 
@@ -284,9 +387,15 @@ class BulkOperator(Tabulator[ModelType]):
                 set(existing_df.columns) & set(df.columns) & set(columns.keys())
             )  # all cols which exist in both dfs and the db model
             - set(self.model_class.updateable_columns)  # no updateable columns
-            - set(primary_key_columns)  # no pk columns
+            - set(primary_key_columns.keys())  # no pk columns
         )  # = all columns that are constant and provided during creation
 
+        for x in existing_df.select_dtypes(include=["datetime64"]).columns.tolist():
+            existing_df.loc[:, x] = existing_df[x].astype(object)
+
+        for x in df.select_dtypes(include=["datetime64"]).columns.tolist():
+            df.loc[:, x] = df[x].astype(object)
+
         return df.merge(
             existing_df,
             how="left",
@@ -305,15 +414,45 @@ class BulkOperator(Tabulator[ModelType]):
         ]
 
         df = df.drop(columns=existing_columns + extra_columns)
-        df = df.dropna(axis="columns", how="all")
-        df = df.dropna()
+        df = df.dropna(axis="index", how="all")
+        df = df.replace({np.nan: None})
+
         return df
 
     def split_by_max_unique_values(
         self, df: pd.DataFrame, columns: Iterable[str], mu: int
-    ) -> Tuple[pd.DataFrame, pd.DataFrame]:
-        df_len = len(df.index)
-        chunk_size = df_len
+    ) -> tuple[pd.DataFrame, pd.DataFrame]:
+        """
+        Splits a DataFrame into two parts based on the maximum allowed unique
+        values in specified columns.
+
+        The function iteratively adjusts the chunk size starting from
+        the entire DataFrame, reducing it until the maximum number of unique
+        values in any of the specified columns within the chunk is ≤ `mu`.
+        The remaining rows form the second part.
+
+        Args:
+            df (pd.DataFrame): The input DataFrame to split.
+            columns (Iterable[str]): Columns to check for the maximum unique values.
+            mu (int): Maximum allowed unique values in any
+                of the specified columns for the first chunk.
+
+        Returns:
+            tuple[pd.DataFrame, pd.DataFrame]:
+                - First element (chunk_df): Largest initial chunk where
+                    columns have ≤ `mu` unique values.
+                - Second element (remaining_df): Remaining rows of the
+                    DataFrame after `chunk_df`.
+
+        Example:
+            If `df` has 100 rows and a column initially has 10 unique values
+            (with `mu=5`), the chunk is reduced to 50 rows.
+            If the chunk then has 6 unique values, it's further
+            reduced to ~41 rows, and so on until all columns in the chunk
+            meet the condition.
+        """
+
+        chunk_size = len(df.index)
         remaining_df = pd.DataFrame()
 
         if chunk_size <= mu:
@@ -363,7 +502,7 @@ class BulkUpserter(BulkOperator[ModelType]):
     def bulk_upsert_chunk(self, df: pd.DataFrame) -> None:
         logger.debug(f"Starting `bulk_upsert_chunk` for {len(df)} rows.")
         columns = db.utils.get_columns(self.model_class)
-        df = df[list(set(columns.keys()) & set(df.columns))]
+        df = df[list(set(columns.keys()) & set(df.columns))].copy()
         existing_df = self.tabulate_existing(df)
         if existing_df.empty:
             logger.debug(f"Inserting {len(df)} rows.")
@@ -389,9 +528,11 @@ class BulkUpserter(BulkOperator[ModelType]):
             insert_df = self.drop_merge_artifacts(
                 df.where(~df["exists"]), extra_columns=["id", "differs", "exists"]
             )
-            update_df = self.drop_merge_artifacts(
-                df.where(df["exists"] & df["differs"]),
-                extra_columns=["differs", "exists"],
+            update_df = (
+                self.drop_merge_artifacts(  # gets rid of metas null columns, fix!
+                    df.where(df["exists"] & df["differs"]),
+                    extra_columns=["differs", "exists"],
+                )
             )
 
             if not insert_df.empty:
@@ -403,28 +544,26 @@ class BulkUpserter(BulkOperator[ModelType]):
 
         self.session.commit()
 
-    def bulk_insert(self, df: pd.DataFrame, **kwargs) -> None:
-        # to_dict returns a more general list[Mapping[Hashable, Unknown]]
+    def bulk_insert(self, df: pd.DataFrame, **kwargs: Any) -> None:
         if "id" in df.columns:
             raise ProgrammingError("You may not insert the 'id' column.")
+
+        # to_dict returns a more general list[Mapping[Hashable, Unknown]]
         m = cast(list[dict[str, Any]], df.to_dict("records"))
 
+        exc = db.insert(self.model_class).execution_options(synchronize_session=False)
+
         try:
-            self.session.execute(
-                db.insert(self.model_class),
-                m,
-                execution_options={"synchronize_session": False},
-            )
+            self.session.execute(exc, m)
         except IntegrityError as e:
             raise self.model_class.NotUnique(*e.args)
 
-    def bulk_update(self, df: pd.DataFrame, **kwargs) -> None:
+    def bulk_update(self, df: pd.DataFrame, **kwargs: Any) -> None:
         # to_dict returns a more general list[Mapping[Hashable, Unknown]]
         m = cast(list[dict[str, Any]], df.to_dict("records"))
+
         self.session.execute(
-            db.update(self.model_class),
-            m,
-            execution_options={"synchronize_session": False},
+            db.update(self.model_class).execution_options(synchronize_session=False), m
         )
 
 
@@ -441,13 +580,18 @@ class BulkDeleter(BulkOperator[ModelType]):
         df = self.merge_existing(df, existing_df)
         df["exists"] = np.where(pd.notnull(df["id"]), True, False)
         delete_df = df.where(df["exists"])
-        delete_df = df[["id"]]
+        id_df = delete_df[["id"]]
+
         excs = []
-        for _, sdf in delete_df.groupby(delete_df.index // self.max_list_length):
-            exc = db.delete(self.model_class).where(self.model_class.id.in_(sdf["id"]))
-            excs.append(exc)
+        for _, sdf in id_df.groupby(id_df.index // self.max_list_length):
+            dexc = db.delete(self.model_class).where(self.model_class.id.in_(sdf["id"]))
+            excs.append(dexc)
 
         for exc in excs:
             self.session.execute(exc, execution_options={"synchronize_session": False})
 
         self.session.commit()
+
+
+class TabulateTransactionsKwargs(abstract.annotations.HasPaginationArgs, total=False):
+    pass
