@@ -1,4 +1,3 @@
-# flake8: noqa
 from ..base import BaseModel as RootBaseModel
 from ..base import (
     BulkDeleter,
@@ -9,10 +8,11 @@ from ..base import (
     Lister,
     Retriever,
     Selecter,
+    TabulateTransactionsKwargs,
+    TabulateVersionsKwargs,
     Tabulator,
 )
 
 
 class BaseModel(RootBaseModel):
     __abstract__ = True
-    table_prefix = "iamc_"
