@@ -1,20 +1,28 @@
-from typing import ClassVar
+from typing import TYPE_CHECKING, ClassVar
 
 import pandas as pd
 from sqlalchemy.exc import NoResultFound
 
+# TODO Import this from typing when dropping Python 3.11
+from typing_extensions import Unpack
+
 from ixmp4 import db
-from ixmp4.data import types
-from ixmp4.data.abstract import iamc as abstract
+from ixmp4.data import abstract, types
 from ixmp4.data.auth.decorators import guard
 from ixmp4.data.db import mixins
 
 from ..unit import Unit
+
+if TYPE_CHECKING:
+    from ixmp4.data.backend.db import SqlAlchemyBackend
+
+from .. import versions
 from . import base
 from .variable import Variable
 
 
 class Measurand(base.BaseModel, mixins.HasCreationInfo):
+    __tablename__ = "iamc_measurand"
     NotFound: ClassVar = abstract.Measurand.NotFound
     NotUnique: ClassVar = abstract.Measurand.NotUnique
     DeletionPrevented: ClassVar = abstract.Measurand.DeletionPrevented
@@ -42,6 +50,20 @@ class Measurand(base.BaseModel, mixins.HasCreationInfo):
     )
 
 
+class MeasurandVersion(versions.DefaultVersionModel):
+    __tablename__ = "iamc_measurand_version"
+    variable__id: types.Integer = db.Column(nullable=False, index=True)
+    unit__id: types.Integer = db.Column(nullable=False, index=True)
+
+    created_at: types.DateTime = db.Column(nullable=True)
+    created_by: types.Username
+
+
+version_triggers = versions.PostgresVersionTriggers(
+    Measurand.__table__, MeasurandVersion.__table__
+)
+
+
 class MeasurandRepository(
     base.Creator[Measurand],
     base.Retriever[Measurand],
@@ -50,12 +72,12 @@ class MeasurandRepository(
 ):
     model_class = Measurand
 
-    def __init__(self, *args, **kwargs) -> None:
-        super().__init__(*args, **kwargs)
+    def __init__(self, *args: "SqlAlchemyBackend") -> None:
+        super().__init__(*args)
 
     @guard("view")
     def get(self, variable_name: str, unit__id: int) -> Measurand:
-        exc: db.sql.Select = (
+        exc = (
             db.select(Measurand)
             .join(Measurand.variable)
             .where(Measurand.unit__id == unit__id)
@@ -68,7 +90,7 @@ class MeasurandRepository(
             raise Measurand.NotFound
 
     def add(self, variable_name: str, unit__id: int) -> Measurand:
-        q_exc: db.sql.Select = db.select(Variable).where(Variable.name == variable_name)
+        q_exc = db.select(Variable).where(Variable.name == variable_name)
         try:
             variable = self.session.execute(q_exc).scalar_one()
         except NoResultFound:
@@ -80,16 +102,16 @@ class MeasurandRepository(
         return measurand
 
     @guard("edit")
-    def create(self, *args, **kwargs) -> Measurand:
-        return super().create(*args, **kwargs)
+    def create(self, *args: Unpack[tuple[str, int]]) -> Measurand:
+        return super().create(*args)
 
-    def select(self, *args, **kwargs) -> db.sql.Select:
+    def select(self) -> db.sql.Select[tuple[Measurand]]:
         return db.select(Measurand)
 
     @guard("view")
-    def list(self, *args, **kwargs) -> list[Measurand]:
-        return super().list(*args, **kwargs)
+    def list(self) -> list[Measurand]:
+        return super().list()
 
     @guard("view")
-    def tabulate(self, *args, **kwargs) -> pd.DataFrame:
-        return super().tabulate(*args, **kwargs)
+    def tabulate(self) -> pd.DataFrame:
+        return super().tabulate()
