@@ -9,6 +9,8 @@ from .. import base
 
 
 class Scalar(base.BaseModel):
+    __tablename__ = "optimization_scalar"
+
     NotFound: ClassVar = abstract.Scalar.NotFound
     NotUnique: ClassVar = abstract.Scalar.NotUnique
     DeletionPrevented: ClassVar = abstract.Scalar.DeletionPrevented
@@ -17,8 +19,8 @@ class Scalar(base.BaseModel):
 
     value: types.Float = db.Column(db.Float, nullable=True, unique=False)
 
-    unit: types.Mapped[Unit | None] = db.relationship()
-    unit__id: types.Mapped[int | None] = db.Column(
+    unit: types.Mapped[Unit] = db.relationship()
+    unit__id: types.Mapped[int] = db.Column(
         db.Integer, db.ForeignKey("unit.id"), index=True
     )
 
