@@ -1,13 +1,17 @@
+from typing import Any
+
 from ixmp4 import db
 from ixmp4.data.db import filters as base
 from ixmp4.data.db.iamc.timeseries import TimeSeries
-from ixmp4.db import filters, utils
+from ixmp4.db import Session, filters, typing_column, utils
 
 from . import Region
 
 
 class BaseIamcFilter(filters.BaseFilter, metaclass=filters.FilterMeta):
-    def join_datapoints(self, exc: db.sql.Select, session=None):
+    def join_datapoints(
+        self, exc: db.sql.Select[tuple[Region]], session: Session | None = None
+    ) -> db.sql.Select[tuple[Region]]:
         if not utils.is_joined(exc, TimeSeries):
             exc = exc.join(TimeSeries, onclause=TimeSeries.region__id == Region.id)
         return exc
@@ -16,25 +20,35 @@ class BaseIamcFilter(filters.BaseFilter, metaclass=filters.FilterMeta):
 class SimpleIamcRegionFilter(
     base.RegionFilter, BaseIamcFilter, metaclass=filters.FilterMeta
 ):
-    def join(self, exc, session=None):
+    def join(
+        self, exc: db.sql.Select[tuple[Region]], session: Session | None = None
+    ) -> db.sql.Select[tuple[Region]]:
         return super().join_datapoints(exc, session)
 
 
 class IamcRegionFilter(base.RegionFilter, BaseIamcFilter, metaclass=filters.FilterMeta):
-    variable: base.VariableFilter
-    unit: base.UnitFilter
+    variable: base.VariableFilter | None = filters.Field(None)
+    unit: base.UnitFilter | None = filters.Field(None)
     run: base.RunFilter = filters.Field(
         default=base.RunFilter(id=None, version=None, is_default=True)
     )
 
-    def join(self, exc, session=None):
+    def join(
+        self, exc: db.sql.Select[tuple[Region]], session: Session | None = None
+    ) -> db.sql.Select[tuple[Region]]:
         return super().join_datapoints(exc, session)
 
 
 class RegionFilter(base.RegionFilter, BaseIamcFilter, metaclass=filters.FilterMeta):
-    iamc: IamcRegionFilter | filters.Boolean | None
+    iamc: IamcRegionFilter | filters.Boolean | None = filters.Field(None)
 
-    def filter_iamc(self, exc, c, v, session=None):
+    def filter_iamc(
+        self,
+        exc: db.sql.Select[tuple[Region]],
+        c: typing_column[Any],  # Any since it is unused
+        v: bool | None,
+        session: Session | None = None,
+    ) -> db.sql.Select[tuple[Region]]:
         if v is None:
             return exc
 
@@ -45,5 +59,7 @@ class RegionFilter(base.RegionFilter, BaseIamcFilter, metaclass=filters.FilterMe
             exc = exc.where(~Region.id.in_(ids))
             return exc
 
-    def join(self, exc, **kwargs):
+    def join(
+        self, exc: db.sql.Select[tuple[Region]], session: Session | None = None
+    ) -> db.sql.Select[tuple[Region]]:
         return exc
