@@ -1,16 +1,11 @@
 from pathlib import Path
-from typing import Generator
 
 from ixmp4.conf import settings
 
-database_dir = settings.storage_directory / "databases"
 
-
-def yield_databases() -> Generator[Path, None, None]:
-    """Yields all local sqlite database files."""
-
-    databases = database_dir.glob("*.sqlite3")
-    return databases
+def get_database_dir() -> Path:
+    """Returns the path to the local sqlite database directory."""
+    return settings.storage_directory / "databases"
 
 
 def get_database_path(name: str) -> Path:
@@ -18,7 +13,7 @@ def get_database_path(name: str) -> Path:
     Does not check whether or not the file actually exists."""
 
     file_name = name + ".sqlite3"
-    return database_dir / file_name
+    return get_database_dir() / file_name
 
 
 def get_dsn(database_path: Path) -> str:
@@ -31,7 +26,4 @@ def search_databases(name: str) -> str | None:
     """Returns a database URI if the desired database exists, otherwise `None`."""
 
     database_path = get_database_path(name)
-    if database_path.exists():
-        return get_dsn(database_path)
-    else:
-        return None
+    return get_dsn(database_path) if database_path.exists() else None
